**** WARNING!
**
** This program is used in the build process to automatically
** create the syspack pack/reindex program.  Do not move it
** from this location!!
** 
****
PARAMETER cSrcDir, cPrgName, cCheckTbl, cLogProg
ON ERROR
*************************************************************************
*  PROGRAM:  CDX2PRG
*  DESCRIP:  Code generator for packing and reindexing tables
*   AUTHOR:  Jason Landry, Exchange Resources
*     DATE:  01-Dec-1995
*  
*  DETAILS
*  -------
*  Given a Path (cSrcDir) and a [path]program to create (cPrgName)
*  this function will create a procedure to pack, pack memo, and
*  reindex every table in that directory.  It checks the date of
*  the generated PRG; if it is older than the date of either
*  this program, or any of the CDX files in the subdirectory, it will
*  regenerate the (cPrgName).
*  This program is intended to be called from a menu built with
*  GENMENUX's *:AUTOVER function.  See GENMENUX documentation for how
*  this feature works.  In short, you should create an AUTOVER
*  menu, and in the comment snippet of the menu pad, put the line
*  DO [path]cdx2prg WITH 'PATH\TO\YOUR\DATA','PATH\AND\cPrgName.PRG'
*  It will scan the path you give it, and create the program that
*  you passed.
*  
*  By passing cCheckTbl in the form of 'yourFunction(^)', the caret
*  will be replaced with the name of the current table, and the
*  code will call the routine you specify for each table.  This 
*  functionality is intended for automatic table-validators such
*  as fixfox etc.  The cCheckTbl function is called *BEFORE* attempting
*  to open the table to (obviously) try to repair any problems before
*  they occur.  The cCheckTbl function *MUST* return .T. if processing
*  can continue, or .F. if it cannot.  The generated program will simply
*  loop past the table if it has a problem.  If one or more problems
*  are encountered, the generated program will return .f.; otherwise,
*  it will return .t.
*
*  MODIFICATIONS
*
*  Date        Notes and Programmer
*  ----------- -----------------------------------------------------------
*  01-Dec-1995 Initial programming finished (JML)
*              Add cCheckTbl parameter to call a prg to check table validity
*			   Modified behavior of ON ERROR to handle lExcl=.f.
*  06-Jun-1996 Added support for thermometer instead of message box.
*  01-Jul-1996 Don't close if ALIAS()='SYSTEM' -- this is a semaphore
*              that needs to remain open at all times.
*  09-Jul-1996 Added thermometer code to avoid external dependencies.
**************************************************************************
PRIVATE afiles,fmask,i,flcount,aflds,j, lReindexAll, cAlias
IF TYPE('cSrcDir')<>"C" OR PARAMETERS()=0
	cSrcDir=SYS(5)+SYS(2003)
ENDIF
IF TYPE('cPrgName')<>'C'
	cPrgName="makecdx.prg"
ENDIF
IF TYPE('cCheckTbl')<>'C'
	cCheckTbl=''
ENDIF
cSrcDir=ALLTRIM(cSrcDir)+IIF(RIGHT(cSrcDir,1)="\","","\")

* Figure out if we need to re-do the (cPrgName)
lRefresh=.f.

IF !FILE(cPrgName) OR DTOC(fdate(sys(16)),1)+FTIME(sys(16))>DTOC(fdate(cPrgName),1)+FTIME(cPrgName)
	* This program is newer than the prgfile
	lRefresh=.t.
ELSE
	cAlias=DTOC(fdate(cPrgName),1)+FTIME(cPrgName)
	FOR i=1 TO ADIR(aCDX,cSrcDir+'*.CDX')
		IF DTOC(aCDX(i,3),1)+aCDX(i,4)>cAlias
			lRefresh=.t.
			EXIT
		ENDIF
	ENDFOR

	IF !lRefresh				&& Still no luck
		=ADIR(aCDX,cPrgName)
		IF aCDX(1,2)<50
			lRefresh=.t.		&& Regenerate if zeroed out
		ENDIF
	ENDIF
	
	IF TYPE('aCDX(1,1)')='C'
		RELEASE aCDX
	ENDIF
ENDIF
IF !lRefresh
	SET MESSAGE TO ('No updates to CDX Rebuilder')
	RETURN
ENDIF

* Update to CDX Rebuilder need to be made
SET MESSAGE TO "Gathering File Information"
cALias=ALIAS()
=ADIR(afiles,cSrcDir+'*.DBF')
SET TEXTMERGE TO (cPrgName)
SET TEXTMERGE ON
\\****************************************************************************
\*   <<PADC(cPrgName,71)>>*
\****************************************************************************
\*   <<SPACE(71)>>*
\*   <<PADC('--=!! WARNING !!=--',71)>>*
\*   <<SPACE(71)>>*
\*   This function was automatically created by the code generator          *
\*   <<PADR(SYS(16)+' '+'on '+DMY(DATE())+' at '+TIME(),71)>>*
\*   PLEASE see documentation in the above program for how to re-generate   *
\*   the code.  Do note attempt to modify this code directly!               *
\*   To force regeneration, delete all the code (make it an empty file)     *
\*   <<SPACE(71)>>*
\****************************************************************************
\PARAMETER pWhichPath, lQuiet
\PRIVATE cErr, afiles,i, cerr, lexcl, nprepack, lFixOK, lNoErrors, nNoPack
\cerr=ON('ERROR')
\IF TYPE('PWHICHPATH')<>'C' OR EMPTY(pWhichPath)
\	pWhichPath=''
\ELSE
\	pWhichPath=ALLTRIM(pWhichPath)+IIF(RIGHT(pWhichPath,1)="\","","\")
\ENDIF
\CLOSE DATA	
\lNoErrors=.t.
\nNoPack=0
\IF ADIR(afiles,pWhichPath+'*.DBF')>0
\	lFixOK=.t.
\	IF !lQuiet
\		DO acttherm WITH "Packing and reindexing","Initializing"
\	ENDIF
\	FOR i=1 TO ALEN(afiles,1)
		IF !EMPTY(cCheckTbl)
\		DELETE FILE (pWhichPath+STRTRAN(afiles(i,1),'.DBF','.CDX')	
\		lFixOK=<<STRTRAN(cCheckTbl,'^','pWhichPath+afiles(i,1)')>>
		ENDIF
\		IF !lFixOK
\			lNoErrors=.f.			
\			LOOP
\		ENDIF
\		lexcl=.T.
\		ON ERROR lexcl=.t.
		* the ON ERROR line used to be IIF(INLIST(error(),1707,114,19),.t.,.f.)
\		USE (pWhichPath+afiles(i,1)) && First clear 'CDX not found' errors 
\		USE (pWhichPath+afiles(i,1)) EXCLUSIVE
\		ON ERROR &cerr
\		IF lexcl
\			IF !lQuiet
\				DO UPDTHERM WITH i/ALEN(aFiles,1)*100,'Packing and reindexing',Proper(juststem(afiles(i,1)))
\			*	set message to ('Performing pack/reindex on '+Proper(juststem(afiles(i,1))))
\			ENDIF
\			nprepack=RECCOUNT()
\			PACK
\			PACK MEMO
\			DO CASE
SELECT 0
FOR i=1 to ALEN(aFiles,1)
	IF afiles(i,1)<>"FOXUSER.DBF"
		afiles(i,1)=lower(STRTRAN(afiles(i,1),".DBF",""))
		USE (cSrcDir+afiles(i,1)) AGAIN ALIAS cTmp
		set message to "Creating reindex code for "+afiles(i,1)
		j=1
		IF !EMPTY(TAG(j))
\				CASE ALIAS()==<<"'"+UPPER(afiles(i,1))+"'">> 
\					DELETE TAG ALL
		DO WHILE !EMPTY(TAG(j))
\					INDEX ON <<(SYS(14,j))>> TAG <<LOWER(TAG(j))+IIF(!EMPTY(SYS(2021,j))," FOR "+SYS(2021,j),"")>>
				j=j+1
		ENDDO
		IF !EMPTY(cLogProg)
\					=<<STRTRAN(cLogProg,'^','pWhichPath+afiles(i,1)')>>
		ENDIF
		ENDIF
	ENDIF
ENDFOR
\			ENDCASE
\		ELSE
\			IF !lQuiet
\				set message to('Cannot pack '+LOWER(juststem(afiles(i,1))))
\			ENDIF
\			nNoPack=nNoPack+1
\		ENDIF
#IF .f.
\		IF ALIAS()<>'SYSTEM'
\			USE
\		ELSE
#ENDIF
\			SELECT 0
#IF .f.
\		ENDIF
#ENDIF
\	ENDFOR
\	set message to(' ')
\ENDIF
\IF !lQuiet
\	DO DEACTHERM
\ENDIF
#IF .f.
\DO sysclean WITH 'RUN'
\SELECT SYSTEM
\USE SYSTEM ORDER TAG SYS_CODE
#ENDIF
\RETURN IIF(lNoErrors,nNoPack,-1)
** The following code encapsulates the thermometer within the
** generated program
TEXT
*:**************************************************************
*:         System: 3-D Thermometer
*:         Author: Darren W. Martin
*:      Copyright (c) 1993, Victoria Computerwise Solutions Ltd.
*: 				  #203 - 661 BURNSIDE RD. EAST
*: 				  VICTORIA, BC  V8T 2X9
*:                PHONE (604)382-9473  FAX (604)386-2469
*:  Last modified: 09/26/93 at 14:53:05
*:
*:  Procs & Fncts: ACTTHERM
*:               : UPDTHERM
*:               : DEACTHERM
*:
*:      Documented 15:02:26                                FoxDoc version 3.00a
*:**************************************************************
*
*!**************************************************************
*!      Procedure: ACTTHERM
*!**************************************************************
PROCEDURE acttherm
PARAMETERS mess_1, mess_2
PRIVATE lcoutput
lcoutput=WOUTPUT()
IF NOT WEXIST("THERM_W")
	DEFINE WINDOW therm_w ;
		AT  0.000, 0.000  ;
		SIZE 10.000,98.400 ;
		FONT "Arial", 8 ;
		NOFLOAT ;
		NOCLOSE ;
		NOMINIMIZE ;
		COLOR RGB(,,,192,192,192)
	MOVE WINDOW therm_w CENTER
ENDIF ( NOT WEXIST("THERM_W") )


IF WVISIBLE("THERM_W")
	ACTIVATE WINDOW therm_w SAME
ELSE
	ACTIVATE WINDOW therm_w NOSHOW
ENDIF ( WVISIBLE("THERM_W") )


IF !EMPTY(mess_1)
	IF TXTWIDTH(mess_1) > 84
		DO WHILE TXTWIDTH(mess_1+"...") > 84
			STORE LEFT(mess_1, LEN(mess_1)-1) TO mess_1
		ENDDO ( TXTWIDTH(mess_1+"...") > 84 )
		STORE mess_1 + "..." TO mess_1
	ELSE
		DO WHILE TXTWIDTH(mess_1) < 84
			STORE mess_1 + " " TO mess_1
		ENDDO ( TXTWIDTH(mess_1) < 84 )
	ENDIF ( TXTWIDTH(mess_1) > 84 )
ELSE
	STORE "" TO mess_1
ENDIF ( !EMPTY(mess_1) )

IF !EMPTY(mess_2)
	IF TXTWIDTH(mess_2) > 84
		DO WHILE TXTWIDTH(mess_2+"...") > 84
			STORE LEFT(mess_2, LEN(mess_2)-1) TO mess_2
		ENDDO ( TXTWIDTH(mess_2+"...") > 84 )
		STORE mess_2 + "..." TO mess_2
	ELSE
		DO WHILE TXTWIDTH(mess_2) < 84
			STORE mess_2 + " " TO mess_2
		ENDDO ( TXTWIDTH(mess_2) < 84 )
	ENDIF ( TXTWIDTH(mess_2) > 84 )
ELSE
	STORE "" TO mess_2
ENDIF ( !EMPTY(mess_2) )

@ 0.308,0.600 TO 0.308,97.800 ;
	PEN 2, 8 ;
	STYLE "1" ;
	COLOR RGB(128,128,128,128,128,128)
@ 0.000,0.000 TO 0.000,98.600 ;
	PEN 2, 8 ;
	STYLE "1" ;
	COLOR RGB(255,255,255,255,255,255)
@ 7.846,7.000 SAY "0          10          20          30          40          50          60          70          80          90       100"  ;
	FONT "Arial", 8 ;
	STYLE "T" ;
	COLOR RGB(128,128,128,,,,)
@ 7.769,6.800 SAY "0          10          20          30          40          50          60          70          80          90       100"  ;
	FONT "Arial", 8 ;
	STYLE "T" ;
	COLOR RGB(255,255,255,,,,)
@ 5.308,7.400 TO 7.462,91.400 ;
	PEN 1, 8 ;
	COLOR RGB(128,128,128,,,,)
@ 5.231,7.200 TO 7.385,91.200 ;
	PEN 1, 8 ;
	COLOR RGB(255,255,255,,,,)
@ 6.385,15.400 TO 7.770,15.400 ;
	PEN 1, 8 ;
	COLOR RGB(128,128,128,128,128,128)
@ 6.385,15.200 TO 7.770,15.200 ;
	PEN 1, 8 ;
	COLOR RGB(255,255,255,255,255,255)
@ 6.385,7.400 TO 7.770,7.400 ;
	PEN 1, 8 ;
	COLOR RGB(128,128,128,128,128,128)
@ 6.385,7.200 TO 7.770,7.200 ;
	PEN 1, 8 ;
	COLOR RGB(255,255,255,255,255,255)
@ 6.385,23.800 TO 7.770,23.800 ;
	PEN 1, 8 ;
	COLOR RGB(128,128,128,128,128,128)
@ 6.385,23.600 TO 7.770,23.600 ;
	PEN 1, 8 ;
	COLOR RGB(255,255,255,255,255,255)
@ 6.385,32.200 TO 7.770,32.200 ;
	PEN 1, 8 ;
	COLOR RGB(128,128,128,128,128,128)
@ 6.385,32.000 TO 7.770,32.000 ;
	PEN 1, 8 ;
	COLOR RGB(255,255,255,255,255,255)
@ 6.385,40.600 TO 7.770,40.600 ;
	PEN 1, 8 ;
	COLOR RGB(128,128,128,128,128,128)
@ 6.385,40.400 TO 7.770,40.400 ;
	PEN 1, 8 ;
	COLOR RGB(255,255,255,255,255,255)
@ 6.385,49.000 TO 7.770,49.000 ;
	PEN 1, 8 ;
	COLOR RGB(128,128,128,128,128,128)
@ 6.385,48.800 TO 7.770,48.800 ;
	PEN 1, 8 ;
	COLOR RGB(255,255,255,255,255,255)
@ 6.385,57.400 TO 7.770,57.400 ;
	PEN 1, 8 ;
	COLOR RGB(128,128,128,128,128,128)
@ 6.385,57.200 TO 7.770,57.200 ;
	PEN 1, 8 ;
	COLOR RGB(255,255,255,255,255,255)
@ 6.385,66.000 TO 7.770,66.000 ;
	PEN 1, 8 ;
	COLOR RGB(128,128,128,128,128,128)
@ 6.385,65.800 TO 7.770,65.800 ;
	PEN 1, 8 ;
	COLOR RGB(255,255,255,255,255,255)
@ 6.385,74.200 TO 7.770,74.200 ;
	PEN 1, 8 ;
	COLOR RGB(128,128,128,128,128,128)
@ 6.385,74.000 TO 7.770,74.000 ;
	PEN 1, 8 ;
	COLOR RGB(255,255,255,255,255,255)
@ 6.385,82.600 TO 7.770,82.600 ;
	PEN 1, 8 ;
	COLOR RGB(128,128,128,128,128,128)
@ 6.385,82.400 TO 7.770,82.400 ;
	PEN 1, 8 ;
	COLOR RGB(255,255,255,255,255,255)
@ 6.385,91.200 TO 7.770,91.200 ;
	PEN 1, 8 ;
	COLOR RGB(128,128,128,128,128,128)
@ 6.385,91.000 TO 7.770,91.000 ;
	PEN 1, 8 ;
	COLOR RGB(255,255,255,255,255,255)
@ 0.000,0.000 TO 10.154,0.000 ;
	PEN 2, 8 ;
	COLOR RGB(255,255,255,255,255,255)
@ 10.000,0.000 TO 10.000,98.600 ;
	PEN 2, 8 ;
	STYLE "1" ;
	COLOR RGB(128,128,128,128,128,128)
@ 0.000,98.200 TO 10.154,98.200 ;
	PEN 2, 8 ;
	COLOR RGB(128,128,128,128,128,128)
@ 0.385,0.600 TO 9.770,0.600 ;
	PEN 2, 8 ;
	COLOR RGB(128,128,128,128,128,128)
@ 9.692,0.600 TO 9.692,98.000 ;
	PEN 2, 8 ;
	STYLE "1" ;
	COLOR RGB(255,255,255,255,255,255)
@ 0.462,97.400 TO 9.693,97.400 ;
	PEN 2, 8 ;
	COLOR RGB(255,255,255,255,255,255)

@ 1.308,7.800 SAY mess_1 ;
	SIZE 1.000,69.167 ;
	FONT "Arial", 8 ;
	STYLE "B" ;
	PICTURE "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"

@ 3.077,7.800 SAY mess_2 ;
	SIZE 1.000,69.167 ;
	FONT "Arial", 8 ;
	STYLE "B" ;
	PICTURE "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"

IF NOT WVISIBLE("THERM_W")
	ACTIVATE WINDOW therm_w
ENDIF ( NOT WVISIBLE("THERM_W") )
IF !EMPTY(lcoutput)
	ACTIVATE WINDOW (lcoutput) SAME NOSHOW
ENDIF
RETURN

*!**************************************************************
*!      Procedure: UPDTHERM
*!**************************************************************
PROCEDURE updtherm
PARAMETERS per_done, mess_1, mess_2
PRIVATE lcoutput

IF per_done <= 0 OR !WEXIST('therm_w')
	RETURN
ENDIF ( per_done <= 0 )

lcoutput=WOUTPUT()
ACTIVATE WINDOW therm_w

STORE IIF(per_done > 100,100,per_done) TO per_done

IF !EMPTY(mess_1)
	IF TXTWIDTH(mess_1) > 84
		DO WHILE TXTWIDTH(mess_1+"...") > 84
			STORE LEFT(mess_1, LEN(mess_1)-1) TO mess_1
		ENDDO ( TXTWIDTH(mess_1+"...") > 84 )
		STORE mess_1 + "..." TO mess_1
	ELSE
		DO WHILE TXTWIDTH(mess_1) < 84
			STORE mess_1 + " " TO mess_1
		ENDDO ( TXTWIDTH(mess_1) < 84 )
	ENDIF ( TXTWIDTH(mess_1) > 84 )
ELSE
	STORE "" TO mess_1
ENDIF ( !EMPTY(mess_1) )

IF !EMPTY(mess_2)
	IF TXTWIDTH(mess_2) > 84
		DO WHILE TXTWIDTH(mess_2+"...") > 84
			STORE LEFT(mess_2, LEN(mess_2)-1) TO mess_2
		ENDDO ( TXTWIDTH(mess_2+"...") > 84 )
		STORE mess_2 + "..." TO mess_2
	ELSE
		DO WHILE TXTWIDTH(mess_2) < 84
			STORE mess_2 + " " TO mess_2
		ENDDO ( TXTWIDTH(mess_2) < 84 )
	ENDIF ( TXTWIDTH(mess_2) > 84 )
ELSE
	STORE "" TO mess_2
ENDIF ( !EMPTY(mess_2) )

@ 1.308,7.800 SAY mess_1 ;
	SIZE 1.000,69.167 ;
	FONT "Arial", 8 ;
	STYLE "B" ;
	PICTURE "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"

@ 3.077,7.800 SAY mess_2 ;
	SIZE 1.000,69.167 ;
	FONT "Arial", 8 ;
	STYLE "B" ;
	PICTURE "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"

STORE 7.200 + (per_done/100 * 83.900) TO r_col

@ 5.231,7.200 FILL TO 7.462,r_col ;
	COLOR RGB(0,0,0,0,128,128)
IF !EMPTY(lcoutput)
	ACTIVATE WINDOW (lcoutput) SAME NOSHOW
ENDIF
RETURN

*!*****************************************************************
*!      Procedure: DEACTHERM
*!*****************************************************************
PROCEDURE deactherm
RELEASE WINDOW therm_w
RETURN

ENDTEXT


ENDIF
USE
SET TEXTMERGE TO
SET TEXTMERGE OFF
SET MESSAGE TO ' '
IF !EMPTY(cALias)
	SELECT (cAlias)
ENDIF